# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator (autorest: 3.9.5, generator: @autorest/python@6.39.0)
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class KnowledgeAgentMessageContentType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of message content."""

    TEXT = "text"
    """Text message content kind."""
    IMAGE = "image"
    """Image message content kind."""


class KnowledgeSourceKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The kind of the knowledge source."""

    SEARCH_INDEX = "searchIndex"
    """A knowledge source that reads data from a Search Index."""
    AZURE_BLOB = "azureBlob"
    """A knowledge source that read and ingest data from Azure Blob Storage to a Search Index."""
